import pytest
from myugpt.code_env import run_code, validate_code
from myugpt.schema import (
    ProgramInputs,
    ProgramOutputs,
    ModelPrediction,
    CodingEnv,
    DatasetFrame,
    Validation,
)
from myugpt.mcts import uct_select, expand, is_terminal, simulate, mcts
from myugpt.schema import CodingEnv, ModelPrediction, Node
from myugpt.gpt import MyuGPT, GPT_SYSTEM
from unittest.mock import MagicMock


# Mock for the Testing
class MockGPT(MyuGPT):
    def step(
        self, env: CodingEnv, temperature: float = 0.5
    ) -> ModelPrediction:
        # Mocking GPT response
        return ModelPrediction(
            thought_process="Mock thought process.",
            code="output = input.strip().upper()",
        )


# Test run_code function
def test_run_code():
    code = """output = input.strip().upper()"""
    inputs = ProgramInputs(data=["  test  ", "hello ", "world "])
    expected_outputs = ProgramOutputs(data=["TEST", "HELLO", "WORLD"])

    outputs = run_code(code, inputs)

    assert (
        outputs == expected_outputs
    ), f"Expected {expected_outputs}, but got {outputs}"


# Test validate_code function
def test_validate_code():
    code = """output = input.strip().upper()"""
    model_pred = ModelPrediction(
        thought_process="Mock thought process.", code=code
    )
    inputs = ProgramInputs(data=["  test  ", "hello ", "world "])
    expected_outputs = ProgramOutputs(data=["TEST", "HELLO", "WORLD"])

    validation = validate_code(model_pred, inputs, expected_outputs)

    assert (
        validation.outputs == expected_outputs
    ), f"Expected {expected_outputs}, but got {validation.outputs}"


def test_uct_select():
    root = Node(
        state=CodingEnv(
            dataset_frame=DatasetFrame(
                problem_statement="Test Problem",
                inputs=ProgramInputs(data=["test"]),
                expected_outputs=ProgramOutputs(data=["TEST"]),
            )
        )
    )
    child1 = Node(state=root.state, parent=root, wins=10, visits=50)
    child2 = Node(state=root.state, parent=root, wins=20, visits=50)

    root.children.append(child1)
    root.children.append(child2)

    best_child = uct_select(root)

    assert best_child == child2, f"Expected {child2}, but got {best_child}"


def test_expand():
    root = Node(
        state=CodingEnv(
            dataset_frame=DatasetFrame(
                problem_statement="Test Problem",
                inputs=ProgramInputs(data=["test"]),
                expected_outputs=ProgramOutputs(data=["TEST"]),
            )
        )
    )
    gpt = MockGPT()

    expand(root, gpt, 1)

    assert len(root.children) > 0, "No children were added during expansion."


def test_is_terminal():
    inputs = ["123"]
    expected_outputs = ["456"]
    validation = Validation(outputs=ProgramOutputs(data=["789"]))
    env = CodingEnv(
        dataset_frame=DatasetFrame(
            problem_statement="Test Problem",
            inputs=ProgramInputs(data=inputs),
            expected_outputs=ProgramOutputs(data=expected_outputs),
        ),
        validation=validation,
    )
    node = Node(state=env)

    assert not is_terminal(node), "Node should not be terminal initially"


def test_simulate():
    inputs = ["test"]
    expected_outputs = ["TEST"]
    validation = Validation(outputs=ProgramOutputs(data=["estt"]))

    env = CodingEnv(
        dataset_frame=DatasetFrame(
            problem_statement="Test Problem",
            inputs=ProgramInputs(data=inputs),
            expected_outputs=ProgramOutputs(data=expected_outputs),
        ),
        validation=validation,
    )

    root = Node(state=env)
    gpt = MockGPT()

    result = simulate(root, gpt)

    assert result > 0, "Simulation result should be positive."


def test_mcts():
    env = CodingEnv(
        dataset_frame=DatasetFrame(
            problem_statement="Test Problem",
            inputs=ProgramInputs(data=["  test  ", "hello ", "world "]),
            expected_outputs=ProgramOutputs(data=["TEST", "HELLO", "WORLD"]),
        )
    )
    gpt = MockGPT()

    final_state = mcts(env, iterations=2, gpt=gpt, expand_size=2)

    assert final_state.score > 0, "Final state score should be positive."



"""CLI interface for myugpt project.
"""

from myugpt.dataset import CodeContestsDataset
from myugpt.gpt import MyuGPT
from myugpt.mcts import mcts
from myugpt.schema import CodingEnv
from myugpt.settings import settings


def main():  # pragma: no cover
    """
    The main function executes on commands:
    `python -m myugpt` and `$ myugpt `.
    """
    myugpt = MyuGPT()
    dataset = CodeContestsDataset()

    frame = dataset[0]

    env = CodingEnv(
        dataset_frame=frame,
    )

    print(env.prompt)

    result = mcts(env, settings.MCTS_ITERS, myugpt, settings.MCTS_EXPAND_SIZE)

    print("=" * 20)
    print("Final Result")
    print("=" * 20)
    print(result)
    print("=" * 20)



import pytest
from pydantic import ValidationError
from myugpt.helper import text_similarity

# Sample texts for testing
text1 = "This is a sample sentence."
text2 = "This is another example sentence."
text3 = "Completely different text."


def test_text_similarity_same_text():
    """Test if the similarity of the same text returns 100.0"""
    assert text_similarity(text1, text1) == 100.0


def test_text_similarity_different_texts_high_similarity():
    """Test if the similarity of two similar texts returns a high similarity score"""
    score = text_similarity(text1, text2)
    assert (
        0 <= score < 100
    )  # Since they are similar, ensure it's less than 100 but non-negative


def test_text_similarity_completely_different_texts():
    """Test if the similarity of two different texts returns a lower similarity score"""
    score = text_similarity(text1, text3)
    assert (
        0 <= score < 100
    )  # Since they are different, ensure it's less than 100 but non-negative


def test_invalid_input_none():
    """Test what happens if one of the inputs is None"""
    with pytest.raises(ValidationError):
        text_similarity(None, text1)


def test_invalid_input_empty_string():
    """Test if the similarity of an empty string and a text returns a non-negative score"""
    score = text_similarity("", text1)
    assert 0 <= score < 100


def test_invalid_input_both_empty_strings():
    """Test if the similarity of two empty strings returns 100"""
    assert text_similarity("", "") == 100.0


if __name__ == "__main__":
    pytest.main()



"""
LLM
"""

import instructor
from openai import OpenAI

# MODEL_NAME = "codellama"
MODEL_NAME = "gpt-4o"
# MODEL_NAME = "mistral:instruct"
# MODEL_NAME = "mixtral:8x7b-instruct-v0.1-q2_K"

openai_client = instructor.patch(
    OpenAI(),
    mode=instructor.Mode.JSON,
)



"""Monte Carlo Tree Search implementation"""

import math
import random

import numpy as np

from myugpt.code_env import validate_code
from myugpt.gpt import MyuGPT
from myugpt.schema import CodingEnv, ModelPrediction, Node


def uct_select(node: Node, exploration_weight=1.4) -> Node:
    # Select a child node with the highest UCT value
    # UCT = (wins/visits) + C * sqrt(log(parent.visits)/visits)
    # C is a constant determining the level of exploration
    assert len(node.children) > 0, "No children to select."
    best_value = float("-inf")
    # best_node = None
    for child in node.children:
        uct_value = child.wins / child.visits + exploration_weight * math.sqrt(
            2 * math.log(node.visits) / child.visits
        )
        if uct_value > best_value:
            best_value = uct_value
            best_node = child
    return best_node


def validate_and_score(
    node: Node,
    action: ModelPrediction,
):
    inputs = node.state.dataset_frame.inputs
    expected_outputs = node.state.dataset_frame.expected_outputs
    # Validate the code and calculate the score
    validation = validate_code(
        action,
        inputs,
        expected_outputs,
    )
    next_state = CodingEnv(
        **node.state.model_dump().copy(),
    )
    next_state.model_predictions.append(
        action,
    )
    next_state.validation = validation
    reward = next_state.score

    return next_state, reward


def expand(node: Node, gpt: MyuGPT, expand_size: int):
    # Add all possible next states as children to the node
    actions = gpt.sample(node.state, num_samples=expand_size)
    node.untried_actions = actions

    for action in node.untried_actions:
        next_state, _ = validate_and_score(
            node,
            action,
        )

        child_node = Node(state=next_state, parent=node)
        node.children.append(child_node)


def is_terminal(node: Node):
    # Check if the node is terminal node (i.e. no more actions possible)
    return np.isclose(node.state.score, 100.0, atol=1e-2)


def simulate(node: Node, gpt: MyuGPT):
    # Simulate a random playout from the node's state until a result is reached
    current_node = node
    while not is_terminal(current_node):
        possible_actions = gpt.sample(
            current_node.state,
        )
        action = random.choice(possible_actions)
        next_state, reward = validate_and_score(
            node,
            action,
        )

        current_node = Node(state=next_state)
        current_node.wins += reward  # Update the reward
    return current_node.wins


def backpropagate(node: Node, result: int):
    # Update the win/visit counts from the node to the root
    while node is not None:
        node.visits += 1
        node.wins += result
        node = node.parent


def mcts(root_env: CodingEnv, iterations: int, gpt: MyuGPT, expand_size: int):
    root = Node(state=root_env)
    expand(root, gpt, expand_size=expand_size)
    for _ in range(iterations):
        # Selection
        leaf = uct_select(root)

        # Expansion
        if not is_terminal(leaf):
            expand(leaf, gpt, expand_size=expand_size)

        # Simulation
        simulation_result = simulate(leaf, gpt)

        # Backpropagation
        backpropagate(leaf, simulation_result)

    # Return the action that leads to the best child of the root
    return max(root.children, key=lambda c: c.wins / c.visits).state



import os

DEFAULT_CODE_CONTESTS = os.path.join(
    os.path.expanduser("~"), "Datasets", "code_contests", "data"
)


class Settings:
    @property
    def CODE_CONTESTS(self) -> str:
        return os.environ.get("CODE_CONTESTS", DEFAULT_CODE_CONTESTS)

    @property
    def MCTS_EXPAND_SIZE(self) -> int:
        return int(os.environ.get("MCTS_EXPAND_SIZE", "1"))

    @property
    def MCTS_ITERS(self) -> int:
        return int(os.environ.get("MCTS_ITERS", "3"))


settings = Settings()



import sys
import traceback

from myugpt.schema import (
    ModelPrediction,
    ProgramInputs,
    ProgramOutputs,
    Validation,
)


def run_code(code: str, inputs: ProgramInputs) -> ProgramOutputs:
    """
    Run the provided Python code with the given inputs and return the output.
    """
    outputs = []
    for input in inputs.data:
        data = ""
        try:
            # Create a dictionary to serve as the local namespace for the
            # exec function
            local_namespace = {
                "input": input,
                "output": "",
            }
            exec(code, {}, local_namespace)
            data_tmp = local_namespace.get("output")
            if data_tmp is None:
                data = ""
            else:
                data = str(data_tmp)
        except Exception:
            print(
                "An error occurred while executing the code:", file=sys.stderr
            )
            data = traceback.format_exc()
        finally:
            print("Output:", data)
            assert isinstance(data, str), "Output must be a string."
            outputs.append(data)

    return ProgramOutputs(data=outputs)


def validate_code(
    model_pred: ModelPrediction,
    inputs: ProgramInputs,
    expected_outputs: ProgramOutputs,
) -> Validation:
    """Run the code with the inputs and compare the outputs"""
    # Run the code
    code = model_pred.code
    actual_outputs = run_code(code, inputs)
    print("Actual Outputs:", actual_outputs)
    print("Expected Outputs:", expected_outputs.data)

    correct_mask = [
        actual == expected
        for actual, expected in zip(actual_outputs.data, expected_outputs.data)
    ]
    all_correct = all(correct_mask)

    # Compare the actual outputs with the expected outputs
    if all_correct:
        return Validation(outputs=expected_outputs)
    else:
        print("The actual outputs do not match the expected outputs.")
        return Validation(outputs=actual_outputs)


if __name__ == "__main__":
    pass



import os
import pytest
from myugpt.settings import Settings


def test_default_values(monkeypatch):
    # Unset environment variables
    monkeypatch.delenv("CODE_CONTESTS", raising=False)
    monkeypatch.delenv("MCTS_EXPAND_SIZE", raising=False)
    monkeypatch.delenv("MCTS_ITERS", raising=False)

    # Create settings instance and assert defaults
    settings = Settings()
    expected_default_path = os.path.join(
        os.path.expanduser("~"), "Datasets", "code_contests", "data"
    )

    assert (
        settings.CODE_CONTESTS == expected_default_path
    ), "Default CODE_CONTESTS path does not match expected"
    assert settings.MCTS_EXPAND_SIZE == 1, "Default MCTS_EXPAND_SIZE is not 1"
    assert settings.MCTS_ITERS == 3, "Default MCTS_ITERS is not 3"


def test_environment_variable_overrides(monkeypatch):
    # Set environment variables
    monkeypatch.setenv("CODE_CONTESTS", "/new/path/to/code_contests")
    monkeypatch.setenv("MCTS_EXPAND_SIZE", "5")
    monkeypatch.setenv("MCTS_ITERS", "10")

    # Create settings instance and assert values
    settings = Settings()

    assert (
        settings.CODE_CONTESTS == "/new/path/to/code_contests"
    ), "CODE_CONTESTS should be overridden by environment variable"
    assert (
        settings.MCTS_EXPAND_SIZE == 5
    ), "MCTS_EXPAND_SIZE should be overridden by environment variable"
    assert (
        settings.MCTS_ITERS == 10
    ), "MCTS_ITERS should be overridden by environment variable"


# Run the tests
if __name__ == "__main__":
    pytest.main()



"""
GPT Coder to solve competitive programming problems
"""

from typing import List

from colorama import Fore, Style

from myugpt.llm import MODEL_NAME, openai_client
from myugpt.schema import CodingEnv, ModelPrediction

GPT_SYSTEM = """You are MyuGPT, an advanced coding chat bot that solves \
competitive programming problems. You will be provided with a problem \
statement (along with the previous list of steps taken to solve the problem, \
if applicable). You will produce the next step as python code to solve the \
problem. Remember to think step by step, you will be given multiple shots!

The input is provided as a string in the global input varriable.
The output is expected as a string in the global output varriable.

```python3
global input: str
global output: str

def main():
    # Write your code here
    pass

main()
```
"""


class MyuGPT:
    def __init__(self, max_history=10):
        self.client = openai_client
        self.previous_messages = []
        self.max_history = max_history

    def step(
        self,
        env: CodingEnv,
        temperature: float = 0.5,
    ) -> ModelPrediction:
        """Step through the coding environment to solve the problem."""
        response = self.client.chat.completions.create(
            model=MODEL_NAME,
            temperature=temperature,
            # max_retries=5,
            stream=True,
            messages=[
                {
                    "role": "user",
                    "content": env.prompt,
                },
                {
                    "role": "system",
                    "content": GPT_SYSTEM,
                },
            ],
            max_tokens=4096,
        )

        desc = ""

        print("Response:")

        print(Fore.GREEN)

        for message in response:
            message_data = message.choices[0].delta.content
            if isinstance(message_data, str):
                desc += message_data
                print(message_data, end="")

        print(Style.RESET_ALL)
        print("=" * 20)

        gpt_prediction = self.client.chat.completions.create(
            model=MODEL_NAME,
            response_model=ModelPrediction,
            temperature=temperature,
            max_retries=2,
            messages=[
                {
                    "role": "user",
                    "content": desc,
                },
                {
                    "role": "system",
                    "content": GPT_SYSTEM,
                },
                {
                    "role": "system",
                    "content": "Generate JSON response\n" + "```json\n\n```",
                },
            ],
            # max_tokens=4096,
        )

        print(gpt_prediction)

        return gpt_prediction

    def sample(
        self,
        env: CodingEnv,
        temperature: float = 0.5,
        num_samples: int = 1,
    ) -> List[ModelPrediction]:
        """Sample the coding environment to solve the problem."""

        result = []

        for _ in range(num_samples):
            result.append(
                self.step(
                    env=env,
                    temperature=temperature,
                )
            )

        return result


if __name__ == "__main__":
    from myugpt.dataset import CodeContestsDataset

    dataset = CodeContestsDataset()
    frame = dataset[0]
    gpt = MyuGPT()

    print(type(frame))

    env = CodingEnv(
        dataset_frame=frame,
    )

    print("env.prompt", env.prompt)

    prediction = gpt.step(env)

    print("prediction", prediction)



import os
from glob import glob
from typing import List, Union

import pandas as pd

from myugpt.schema import DatasetFrame, ProgramInputs, ProgramOutputs
from myugpt.settings import settings

CODE_CONTESTS_DATASETS = glob(
    os.path.join(settings.CODE_CONTESTS, "*.parquet")
)


class Dataset:
    def __init__(self):
        raise NotImplementedError()

    def __len__(self) -> int:
        raise NotImplementedError()

    def __getitem__(self, idx) -> Union[DatasetFrame, List[DatasetFrame]]:
        raise NotImplementedError()

    def __iter__(self):
        raise NotImplementedError()

    def __repr__(self):
        return f"Dataset with {len(self)} items"

    def __str__(self):
        return self.__repr__()


class CodeContestsDataset(Dataset):
    def __init__(
        self,
        dataset_path: str = CODE_CONTESTS_DATASETS[0],
    ):
        self.dataset_path = dataset_path
        self.df = pd.read_parquet(self.dataset_path, engine="pyarrow")

    def __len__(self) -> int:
        return len(self.df)

    def __getitem__(self, idx) -> DatasetFrame:
        # if idx is a slice
        # if isinstance(idx, slice):
        #     return [
        #         self[sub_idx] for sub_idx in range(*idx.indices(len(self)))
        #     ]

        frame = self.df.iloc[idx]

        program_inputs = ProgramInputs(
            data=frame["public_tests"]["input"],
        )
        program_outputs = ProgramOutputs(
            data=frame["public_tests"]["output"],
        )

        return DatasetFrame(
            problem_statement=frame["description"],
            inputs=program_inputs,
            expected_outputs=program_outputs,
        )

    def __iter__(self):
        for idx in range(len(self)):
            yield self[idx]


if __name__ == "__main__":
    dataset = CodeContestsDataset()
    print(dataset)
    frame = dataset[0]
    print(frame)



from typing import List

from pydantic import validate_call
from scipy.spatial.distance import cosine

from myugpt.llm import openai_client


@validate_call
def get_embedding(
    text: str, model: str = "text-embedding-3-small"
) -> List[float]:
    text = text.replace("\n", " ")
    return (
        openai_client.embeddings.create(input=[text], model=model)
        .data[0]
        .embedding
    )


@validate_call
def text_similarity(
    text1: str,
    text2: str,
) -> float:
    """Calculate the text similarity (0 to 100)
    Use text embedding to calculate the similarity
    """

    # If the texts are the same, return 1
    if text1 == text2:
        return 100.0

    if len(text1) == 0 or len(text2) == 0:
        return 0.0

    # Compute the embeddings
    text1_embedding = get_embedding(text1)
    text2_embedding = get_embedding(text2)

    # Compute the cosine similarity
    similarity = cosine(text1_embedding, text2_embedding)

    return similarity * 100



"""
Models
"""

import ast
from typing import List

from pydantic import BaseModel, field_validator

from myugpt.helper import text_similarity


class ProgramInputs(BaseModel):
    """Program Input"""

    data: List[str]


class ProgramOutputs(BaseModel):
    """Program Output"""

    data: List[str]


class DatasetFrame(BaseModel):
    """Dataset Frame"""

    problem_statement: str
    inputs: ProgramInputs
    expected_outputs: ProgramOutputs

    def __str__(self):
        rep = "Problem Statement: " + self.problem_statement + "\n"
        rep += "=============\n"
        for index, (inp, out) in enumerate(
            zip(self.inputs.data, self.expected_outputs.data)
        ):
            rep += f"Input[{index}]:\n" + inp + "\n"
            rep += f"ExpectedOutputs[{index}]:\n" + out + "\n"
            rep += "=============\n"

        return rep


class ModelPrediction(BaseModel):
    """Model Prediction"""

    thought_process: str
    code: str
    # code: Annotated[
    #     str,
    #     BeforeValidator(
    #         llm_validator(
    #             "Write a valid Python code, "
    #             + "don't worry about varriable declatation or "
    #             + "logical errors, "
    #             + "don't worry about missing the necessary context and "
    #             + "structure to be considered valid Python code",
    #             openai_client,
    #         )
    #     ),
    # ]

    @field_validator("code")
    def is_valid_python(cls, v):
        try:
            ast.parse(v)
        except SyntaxError as se:
            raise ValueError(
                f"Invalid Python code:\n"
                f"SyntaxError:"
                f"{se.msg}"
                f"at line {se.lineno}"
            )
        return v

    # score: Annotated[
    #     float,
    #     BeforeValidator(
    #         llm_validator(
    #             "Score the correctness of your code (0 to 100)",
    #             openai_client,
    #         )
    #     ),
    # ]

    def __str__(self):
        rep = "ThoughtProcess:\n" + self.thought_process + "\n"
        rep += "Code:\n" + self.code + "\n"
        rep += "=============\n"
        # for index, out in enumerate(self.predicted_outputs.data):
        #     rep += f"PredictedOutputs[{index}]:\n" + out + "\n"
        #     rep += "=============\n"
        rep += "=============\n"
        # rep += "Score:\n" + str(self.score) + "\n"
        # rep += "=============\n"
        return rep


class Validation(BaseModel):
    """Validation"""

    outputs: ProgramOutputs


class CodingEnv(BaseModel):
    """Coding Environment"""

    # From Dataset
    dataset_frame: DatasetFrame

    # From Model
    model_predictions: List[ModelPrediction] = []

    # From Validation
    validation: Validation = Validation(outputs=ProgramOutputs(data=[]))

    @property
    def prompt(self):
        """Convert the code env to a prompt"""
        prompt = str(self.dataset_frame)
        prompt += "=============\n"
        for index, (model_prediction, validation) in enumerate(
            zip(self.model_predictions, self.validation.outputs.data)
        ):
            prompt += f"Code[{index}]:\n" + str(model_prediction) + "\n"
            prompt += "=============\n"
            prompt += f"Output[{index}]:\n" + str(validation) + "\n"
            prompt += "=============\n"
        return prompt

    @property
    def score(self):
        """Calculate the score"""
        res = 0
        model_prediction_list = self.dataset_frame.expected_outputs.data
        validation_list = self.validation.outputs.data

        assert len(model_prediction_list) == len(
            validation_list
        ), f"{len(model_prediction_list)} != {len(validation_list)}"

        for model_prediction, validation in zip(
            model_prediction_list, validation_list
        ):
            frame_score = text_similarity(
                model_prediction,
                validation,
            )
            assert 0 <= frame_score <= 100
            res += frame_score
        res = res / len(model_prediction_list)
        return res


# @no_type_check
class Node(BaseModel):
    """Node for MCTS"""

    state: CodingEnv  # type: ignore

    parent: "Node" = None  # type: ignore
    children: List["Node"] = []  # type: ignore
    wins: float = 0
    visits: int = 1
    untried_actions: List[ModelPrediction] = []



"""
myugpt base module.

This is the principal module of the myugpt project.
here you put your main classes and objects.

Be creative! do whatever you want!

If you want to replace this with a Flask application run:

    $ make init

and then choose `flask` as template.
"""

# example constant variable
NAME = "myugpt"



from myugpt.schema import (
    ModelPrediction,
    ProgramInputs,
    ProgramOutputs,
    Validation,
)
from myugpt.code_env import run_code, validate_code

# sample code for testing
test_code = """
output = input[::-1];  # reverse the input string
"""


def test_run_code():
    inputs = ProgramInputs(data=["hello", "world"])
    expected_outputs = ProgramOutputs(data=["olleh", "dlrow"])

    outputs = run_code(test_code, inputs)

    assert (
        outputs == expected_outputs
    ), f"Expected {expected_outputs} but got {outputs}"


def test_validate_code_correct():
    model_pred = ModelPrediction(
        thought_process="reverse input", code=test_code
    )
    inputs = ProgramInputs(data=["hello", "world"])
    expected_outputs = ProgramOutputs(data=["olleh", "dlrow"])

    validation = validate_code(model_pred, inputs, expected_outputs)

    assert (
        validation.outputs == expected_outputs
    ), f"Expected {expected_outputs} but got {validation.outputs}"


def test_validate_code_incorrect():
    incorrect_code = """
output = input  # no change to the input string
"""
    model_pred = ModelPrediction(
        thought_process="no reverse", code=incorrect_code
    )
    inputs = ProgramInputs(data=["hello", "world"])
    expected_outputs = ProgramOutputs(data=["olleh", "dlrow"])
    actual_outputs = ProgramOutputs(data=["hello", "world"])

    validation = validate_code(model_pred, inputs, expected_outputs)

    assert (
        validation.outputs == actual_outputs
    ), f"Expected {actual_outputs} but got {validation.outputs}"


if __name__ == "__main__":
    import pytest

    pytest.main()



# test_schema.py

import pytest
from pydantic import ValidationError
from myugpt.schema import (
    ProgramInputs,
    ProgramOutputs,
    DatasetFrame,
    ModelPrediction,
    Validation,
    CodingEnv,
)


def test_program_inputs():
    data = ["input1", "input2"]
    inputs = ProgramInputs(data=data)
    assert inputs.data == data


def test_program_outputs():
    data = ["output1", "output2"]
    outputs = ProgramOutputs(data=data)
    assert outputs.data == data


def test_dataset_frame():
    problem_statement = "Sample Problem"
    inputs = ProgramInputs(data=["input1", "input2"])
    outputs = ProgramOutputs(data=["output1", "output2"])
    dataset_frame = DatasetFrame(
        problem_statement=problem_statement,
        inputs=inputs,
        expected_outputs=outputs,
    )
    assert dataset_frame.problem_statement == problem_statement
    assert dataset_frame.inputs == inputs
    assert dataset_frame.expected_outputs == outputs


def test_model_prediction_valid_code():
    thought_process = "Sample thought process"
    code = "print('Hello, world!')"
    prediction = ModelPrediction(thought_process=thought_process, code=code)
    assert prediction.thought_process == thought_process
    assert prediction.code == code


def test_model_prediction_invalid_code():
    thought_process = "Sample thought process"
    code = "print('Hello, world!'"  # Missing closing parenthesis
    with pytest.raises(ValidationError):
        ModelPrediction(thought_process=thought_process, code=code)


def test_validation():
    outputs = ProgramOutputs(data=["output1", "output2"])
    validation = Validation(outputs=outputs)
    assert validation.outputs == outputs


def test_coding_env():
    problem_statement = "Sample Problem"
    inputs = ProgramInputs(data=["input1", "input2"])
    expected_outputs = ProgramOutputs(data=["output1", "output2"])
    dataset_frame = DatasetFrame(
        problem_statement=problem_statement,
        inputs=inputs,
        expected_outputs=expected_outputs,
    )
    model_predictions = [
        ModelPrediction(
            thought_process="Sample thought process",
            code="print('Hello, world!')",
        )
    ]
    validation = Validation(
        outputs=ProgramOutputs(data=["output1", "output2"])
    )
    coding_env = CodingEnv(
        dataset_frame=dataset_frame,
        model_predictions=model_predictions,
        validation=validation,
    )
    assert coding_env.dataset_frame == dataset_frame
    assert coding_env.model_predictions == model_predictions
    assert coding_env.validation == validation


def test_coding_env_prompt():
    problem_statement = "Sample Problem"
    inputs = ProgramInputs(data=["input1", "input2"])
    outputs = ProgramOutputs(data=["output1", "output2"])
    dataset_frame = DatasetFrame(
        problem_statement=problem_statement,
        inputs=inputs,
        expected_outputs=outputs,
    )
    coding_env = CodingEnv(dataset_frame=dataset_frame)
    prompt = coding_env.prompt
    assert problem_statement in prompt
    assert "input1" in prompt
    assert "output1" in prompt


def test_coding_env_score():
    problem_statement = "Sample Problem"
    inputs = ProgramInputs(data=["input1", "input2"])
    expected_outputs = ProgramOutputs(data=["output1", "output2"])
    dataset_frame = DatasetFrame(
        problem_statement=problem_statement,
        inputs=inputs,
        expected_outputs=expected_outputs,
    )
    model_predictions = [
        ModelPrediction(
            thought_process="Sample thought process", code="print('output1')"
        )
    ]
    validation = Validation(
        outputs=ProgramOutputs(data=["output1", "output2"])
    )
    coding_env = CodingEnv(
        dataset_frame=dataset_frame,
        model_predictions=model_predictions,
        validation=validation,
    )
    score = coding_env.score
    assert (
        0.0 <= score <= 100.0
    ), f"Expected score between 0 and 100, but got {score}"
